@@ -82,6 +82,24 @@ CfCmd.new do
       info: "Defines a server or location as a pubsub endpoint. For long-polling, GETs subscribe. and POSTs publish. For Websockets, publishing data on a connection does not yield a channel metadata response. Without additional configuration, this turns a location into an echo server.",
       uri: '#pubsub-endpoint'
   
+  nchan_subscriber_info [:loc],
+      :nchan_subscriber_info_directive,
+      :loc_conf,
+      args: 0,
+      
+      group: "pubsub",
+      tags: ["subscriber", "debug"],
+      info: "A subscriber location for debugging the state of subscribers on a given channel. The subscribers of the channel specified by `nchan_channel_id` evaluate `nchan_subscriber_info_string` and send it back to the requested on this location. This is useful to see where subscribers are in an Nchan cluster, as well as debugging subscriber connection issues."
+  
+  nchan_subscriber_info_string [:srv, :loc],
+      :ngx_http_set_complex_value_slot,
+      [:loc_conf, :subscriber_info_string],
+      args: 1,
+      
+      group: "pubsub",
+      tags: ["subscriber", "debug"],
+      default: "$nchan_subscriber_type $remote_addr:$remote_port $http_user_agent $server_name $request_uri $pid",
+      info: "this string is evaluated by each subscriber on a given channel and sent to the requester of a `nchan_subscriber_info` location"
   
   nchan_longpoll_multipart_response [:srv, :loc, :if],
       :nchan_set_longpoll_multipart,
@@ -103,7 +121,46 @@ CfCmd.new do
       tags: ['subscriber-eventsource'],
       default: "(none)",
       info: "Set the EventSource `event:` line to this value. When used in a publisher location, overrides the published message's `X-EventSource-Event` header and associates the message with the given value. When used in a subscriber location, overrides all messages' associated `event:` string with the given value."
+  
+  nchan_eventsource_ping_comment [:srv, :loc, :if],
+      :ngx_conf_set_str_slot_no_newlines,
+      [:loc_conf, :"eventsource_ping.comment"],
+      args: 1,
       
+      group: "pubsub",
+      tags: ['subscriber-eventsource'],
+      default: "(empty)",
+      info: "Set the EventSource comment `: ...` line for periodic pings from server to client. Newlines are not allowed. If empty, no comment is sent with the ping."
+  
+  nchan_eventsource_ping_event [:srv, :loc, :if],
+      :ngx_conf_set_str_slot_no_newlines,
+      [:loc_conf, :"eventsource_ping.event"],
+      args: 1,
+      
+      group: "pubsub",
+      tags: ['subscriber-eventsource'],
+      default: "ping",
+      info: "Set the EventSource `event:` line for periodic pings from server to client. Newlines are not allowed. If empty, no event type is sent with the ping."
+  
+  nchan_eventsource_ping_data [:srv, :loc, :if],
+      :ngx_conf_set_str_slot_no_newlines,
+      [:loc_conf, :"eventsource_ping.data"],
+      args: 1,
+      
+      group: "pubsub",
+      tags: ['subscriber-eventsource'],
+      default: "(empty)",
+      info: "Set the EventSource `data:` line for periodic pings from server to client. Newlines are not allowed. If empty, no data is sent with the ping."
+  
+    nchan_eventsource_ping_interval [:srv, :loc, :if],
+      :ngx_conf_set_sec_slot,
+      [:loc_conf, :"eventsource_ping.interval"],
+      
+      group: "pubsub",
+      tags: ['subscriber-eventsource'],
+      value: "<number> (seconds)",
+      default: "0 (none)",
+      info: "Interval for sending ping messages to EventSource subscribers. Disabled by default."
   
   nchan_subscriber [:srv, :loc, :if],
       :nchan_subscriber_directive,
@@ -391,14 +448,27 @@ CfCmd.new do
       
       group: "storage",
       tags: ['redis'],
-      value: ["distributed", "backup"],
+      value: ["distributed", "backup", "nostore"],
       default: "distributed",
       info: <<-EOS.gsub(/^ {8}/, '')
         The mode of operation of the Redis server. In `distributed` mode, messages are published directly to Redis, and retrieved in real-time. Any number of Nchan servers in distributed mode can share the Redis server (or cluster). Useful for horizontal scalability, but suffers the latency penalty of all message publishing going through Redis first.
         
         In `backup` mode, messages are published locally first, then later forwarded to Redis, and are retrieved only upon chanel initialization. Only one Nchan server should use a Redis server (or cluster) in this mode. Useful for data persistence without sacrificing response times to the latency of a round-trip to Redis.
+        
+        In `nostore` mode, messages are published as in `distributed` mode, but are not stored. Thus Redis is used to broadcast messages to many Nchan instances with no delivery guarantees during connection failure, and only local in-memory storage. This means that there are also no message delivery guarantees for subscribers switching from one Nchan instance to another connected to the same Redis server or cluster. Nostore mode increases Redis publishing capacity by an order of magnitude.
       EOS
   
+  nchan_redis_nostore_fastpublish [:main, :srv, :upstream],
+      :ngx_conf_set_flag_slot,
+      [:loc_conf, :"redis.nostore_fastpublish"],
+      
+      group: "storage",
+      tags: ['redis', 'publisher'],
+      value: [:on, :off],
+      default: :off,
+      info: "Increases publishing capacity by 2-3x for Redis nostore mode at the expense of inaccurate subscriber counts in the publisher response."
+  
+  
   nchan_use_redis [:main, :srv, :loc],
       :ngx_conf_enable_redis,
       [:loc_conf, :"redis.url_enabled"],
@@ -410,7 +480,7 @@ CfCmd.new do
       info: "Use of this command is discouraged in favor of (`nchan_redis_pass`)[#nchan_redis_pass]. Use Redis for message storage at this location.",
       uri: "#connecting-to-a-redis-server"
   
-  nchan_redis_ping_interval [:main, :srv, :loc],
+  nchan_redis_ping_interval [:main, :srv, :upstream, :loc],
       :ngx_conf_set_sec_slot,
       [:loc_conf, :"redis.ping_interval"],
       
@@ -457,10 +527,11 @@ CfCmd.new do
       default: "cpu",
       info: "This tweaks whether [effect replication](https://redis.io/commands/eval#replicating-commands-instead-of-scripts) is enabled. Optimizing for CPU usage enables effect replication, costing additional bandwidth (between 1.2 and 2 times more) between all master->slave links. Optimizing for bandwidth increases CPU load on slaves, but keeps outgoing bandwidth used for replication the same as the incoming bandwidth on Master."
   
-  nchan_redis_namespace [:main, :srv, :upstream], 
-      :ngx_conf_set_redis_namespace_slot,
+  nchan_redis_namespace [:main, :srv, :upstream, :loc], 
+      :ngx_conf_set_str_slot,
       [:loc_conf, :"redis.namespace"],
       args: 1,
+      post_handler: "ngx_conf_process_redis_namespace_slot",
       
       group: "storage",
       value: "<string>",
@@ -528,7 +599,7 @@ CfCmd.new do
       value: "<string>",
       tags: ['publisher', 'subscriber'],
       default: "$http_origin",
-      info: "Set the [Cross-Origin Resource Sharing (CORS)](https://developer.mozilla.org/en-US/docs/Web/HTTP/Access_control_CORS) `Access-Control-Allow-Origin` header to this value. If the incoming request's `Origin` header does not match this value, respond with a `403 Forbidden`."
+      info: "Set the [Cross-Origin Resource Sharing (CORS)](https://developer.mozilla.org/en-US/docs/Web/HTTP/Access_control_CORS) `Access-Control-Allow-Origin` header to this value. If the incoming request's `Origin` header does not match this value, respond with a `403 Forbidden`. Multiple origins can be provided in a single argument separated with a space."
 
   nchan_access_control_allow_credentials [:main, :srv, :loc, :if], 
       :ngx_conf_set_flag_slot,
@@ -660,8 +731,8 @@ CfCmd.new do
       group: "security",
       tags: ['publisher', 'subscriber', 'channel-id' ],
       value: "<number>",
-      default: 512,
-      info: "Maximum permissible channel id length (number of characters). Longer ids will be truncated."
+      default: 1024,
+      info: "Maximum permissible channel id length (number of characters). This settings applies to ids before they may be split by the `nchan_channel_id_split_delimiter` Requests with a channel id that is too long will receive a `403 Forbidden` response."
   
   nchan_max_channel_subscribers [:main, :srv, :loc],
       :ngx_conf_set_num_slot,
@@ -691,6 +762,49 @@ CfCmd.new do
       default: "memory",
       info: "Development directive to completely replace default storage engine. Don't use unless you are an Nchan developer."
   
+  nchan_benchmark [:loc],
+      :nchan_benchmark_directive,
+      [:loc_conf],
+      group: "development",
+      undocumented: true,
+      args: 0
+  nchan_benchmark_time [:loc],
+      :ngx_conf_set_sec_slot,
+      [:loc_conf, "benchmark.time"],
+      group: "development",
+      undocumented: true
+  nchan_benchmark_messages_per_channel_per_minute [:loc],
+      :ngx_conf_set_num_slot,
+      [:loc_conf, "benchmark.msgs_per_minute"],
+      group: "development",
+      undocumented: true
+  nchan_benchmark_message_padding_bytes [:loc],
+      :ngx_conf_set_num_slot,
+      [:loc_conf, "benchmark.msg_padding"],
+      group: "development",
+      undocumented: true
+  nchan_benchmark_channels [:loc],
+      :ngx_conf_set_num_slot,
+      [:loc_conf, "benchmark.channels"],
+      group: "development",
+      undocumented: true
+  nchan_benchmark_subscribers_per_channel [:loc],
+      :ngx_conf_set_num_slot,
+      [:loc_conf, "benchmark.subscribers_per_channel"],
+      group: "development",
+      undocumented: true
+  nchan_benchmark_subscriber_distribution [:loc],
+      :nchan_benchmark_subscriber_distribution_directive,
+      [:loc_conf, "benchmark.subscriber_distribution"],
+      group: "development",
+      undocumented: true
+  nchan_benchmark_publisher_distribution [:loc],
+      :nchan_benchmark_publisher_distribution_directive,
+      [:loc_conf, "benchmark.publisher_distribution"],
+      group: "development",
+      undocumented: true
+
+  
   push_min_message_buffer_length [:srv, :loc, :if],
       :nchan_ignore_obsolete_setting,
       :loc_conf,
