@@ -8,11 +8,19 @@
 #include <zlib.h>
 #endif
 
+static char *nchan_set_complex_value_array(ngx_conf_t *cf, ngx_command_t *cmd, void *conf, nchan_complex_value_arr_t *chid);
+static ngx_int_t set_complex_value_array_size1(ngx_conf_t *cf, nchan_complex_value_arr_t *chid, char *val);
+
 static ngx_str_t      DEFAULT_CHANNEL_EVENT_STRING = ngx_string("$nchan_channel_event $nchan_channel_id");
 
+static ngx_str_t      DEFAULT_SUBSCRIBER_INFO_STRING = ngx_string("$nchan_subscriber_type $remote_addr:$remote_port $http_user_agent $server_name $request_uri $pid");
+
 nchan_store_t   *default_storage_engine = &nchan_store_memory;
+ngx_flag_t       global_nchan_enabled = 0;
 ngx_flag_t       global_redis_enabled = 0;
 ngx_flag_t       global_zstream_needed = 0;
+ngx_flag_t       global_benchmark_enabled = 0;
+void            *global_owner_cycle = NULL;
 
 #define MERGE_UNSET_CONF(conf, prev, unset, default)         \
 if (conf == unset) {                                         \
@@ -22,11 +30,25 @@ if (conf == unset) {                                         \
 #define MERGE_CONF(cf, prev_cf, name) if((cf)->name == NULL) { (cf)->name = (prev_cf)->name; }
 
 static ngx_int_t nchan_init_module(ngx_cycle_t *cycle) {
+  if(global_owner_cycle && global_owner_cycle != ngx_cycle) {
+    global_nchan_enabled = 0;
+    global_redis_enabled = 0;
+    global_zstream_needed = 0;
+    global_benchmark_enabled = 0;
+  }
+  global_owner_cycle = (void *)ngx_cycle;
+  
+  if(!global_nchan_enabled) {
+    return NGX_OK;
+  }
   ngx_core_conf_t         *ccf = (ngx_core_conf_t *) ngx_get_conf(cycle->conf_ctx, ngx_core_module);
   nchan_worker_processes = ccf->worker_processes;
   
   //initialize storage engines
   nchan_store_memory.init_module(cycle);
+  if(global_benchmark_enabled) {
+    nchan_benchmark_init_module(cycle);
+  }
   if(global_redis_enabled) {
     nchan_store_redis.init_module(cycle);
   }
@@ -34,6 +56,9 @@ static ngx_int_t nchan_init_module(ngx_cycle_t *cycle) {
 }
 
 static ngx_int_t nchan_init_worker(ngx_cycle_t *cycle) {
+  if(!global_nchan_enabled) {
+    return NGX_OK;
+  }
   if (ngx_process != NGX_PROCESS_WORKER && ngx_process != NGX_PROCESS_SINGLE) {
     //not a worker, stop initializing stuff.
     return NGX_OK;
@@ -42,6 +67,9 @@ static ngx_int_t nchan_init_worker(ngx_cycle_t *cycle) {
   if(nchan_store_memory.init_worker(cycle)!=NGX_OK) {
     return NGX_ERROR;
   }
+  if(global_benchmark_enabled) {
+    nchan_benchmark_init_worker(cycle);
+  }
   
   if(global_redis_enabled && nchan_store_redis.init_worker(cycle)!=NGX_OK) {
     return NGX_ERROR;
@@ -53,7 +81,14 @@ static ngx_int_t nchan_init_worker(ngx_cycle_t *cycle) {
   return NGX_OK;
 }
 
+static ngx_int_t nchan_preconfig(ngx_conf_t *cf) {
+  global_owner_cycle = (void *)ngx_cycle;
+  global_nchan_enabled = 0;
+  return nchan_add_variables(cf);
+}
+
 static ngx_int_t nchan_postconfig(ngx_conf_t *cf) {
+  global_owner_cycle = (void *)ngx_cycle;
   if(nchan_store_memory.init_postconfig(cf)!=NGX_OK) {
     return NGX_ERROR;
   }
@@ -68,6 +103,8 @@ static ngx_int_t nchan_postconfig(ngx_conf_t *cf) {
   }
 #endif
   
+  global_nchan_enabled = 1;
+  
   return NGX_OK;
 }
 
@@ -103,6 +140,7 @@ static void *nchan_create_srv_conf(ngx_conf_t *cf) {
   scf->redis.optimize_target = NCHAN_REDIS_OPTIMIZE_UNSET;
   scf->redis.master_weight = NGX_CONF_UNSET;
   scf->redis.slave_weight = NGX_CONF_UNSET;
+  scf->upstream_nchan_loc_conf = NULL;
   return scf;
 }
 
@@ -150,6 +188,9 @@ static void *nchan_create_loc_conf(ngx_conf_t *cf) {
   
   lcf->subscriber_first_message=NCHAN_SUBSCRIBER_FIRST_MESSAGE_UNSET;
   
+  lcf->subscriber_info_string=NULL;
+  lcf->subscriber_info_location=NGX_CONF_UNSET;
+  
   lcf->subscriber_timeout=NGX_CONF_UNSET;
   lcf->subscribe_only_existing_channel=NGX_CONF_UNSET;
   lcf->redis_idle_channel_cache_timeout=NGX_CONF_UNSET;
@@ -160,6 +201,8 @@ static void *nchan_create_loc_conf(ngx_conf_t *cf) {
   
   lcf->websocket_ping_interval=NGX_CONF_UNSET;
   
+  lcf->eventsource_ping.interval=NGX_CONF_UNSET;
+  
   lcf->msg_in_etag_only = NGX_CONF_UNSET;
   
   lcf->allow_origin = NULL;
@@ -185,10 +228,19 @@ static void *nchan_create_loc_conf(ngx_conf_t *cf) {
   lcf->redis.ping_interval = NGX_CONF_UNSET;
   lcf->redis.upstream_inheritable=NGX_CONF_UNSET;
   lcf->redis.storage_mode = REDIS_MODE_CONF_UNSET;
+  lcf->redis.nostore_fastpublish = NGX_CONF_UNSET;
   lcf->redis.privdata = NULL;
   lcf->redis.nodeset = NULL;
   
   lcf->request_handler = NULL;
+  
+  lcf->benchmark.time = NGX_CONF_UNSET;
+  lcf->benchmark.msgs_per_minute = NGX_CONF_UNSET;
+  lcf->benchmark.msg_padding = NGX_CONF_UNSET;
+  lcf->benchmark.channels = NGX_CONF_UNSET;
+  lcf->benchmark.subscribers_per_channel = NGX_CONF_UNSET;
+  lcf->benchmark.subscriber_distribution = NCHAN_BENCHMARK_SUBSCRIBER_DISTRIBUTION_UNSET;
+  lcf->benchmark.publisher_distribution = NCHAN_BENCHMARK_PUBLISHER_DISTRIBUTION_UNSET;
   return lcf;
 }
 
@@ -277,9 +329,29 @@ static char *nchan_setup_handler(ngx_conf_t *cf, ngx_int_t (*handler)(ngx_http_r
   return NGX_CONF_OK;
 }
 
+static nchan_loc_conf_t *nchan_loc_conf_get_upstream_lcf(nchan_loc_conf_t *conf, nchan_loc_conf_t *prev) {
+  nchan_redis_conf_t *rcf = &conf->redis, *prev_rcf = &prev->redis;
+  if(rcf->upstream == prev_rcf->upstream || rcf->upstream == NULL) {
+    //same or no upstream, so don't bother
+    return NULL;
+  }
+  else {
+    assert(rcf->upstream);
+    nchan_srv_conf_t      *upstream_scf = ngx_http_conf_upstream_srv_conf(rcf->upstream, ngx_nchan_module);
+    if(upstream_scf && upstream_scf->upstream_nchan_loc_conf) {
+      return upstream_scf->upstream_nchan_loc_conf;
+    }
+    else {
+      //ngx_conf_log_error(NGX_LOG_WARN, cf, 0, "nchan upstream srv_conf loc_conf ptr is null");
+      return NULL;
+    }
+  }
+}
+
+
 static char * nchan_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child) {
   nchan_loc_conf_t       *prev = parent, *conf = child;
-  
+  nchan_loc_conf_t       *up = nchan_loc_conf_get_upstream_lcf(conf, prev);
   //publisher types
   ngx_conf_merge_bitmask_value(conf->pub.http, prev->pub.http, 0);
   ngx_conf_merge_bitmask_value(conf->pub.websocket, prev->pub.websocket, 0);
@@ -315,7 +387,21 @@ static char * nchan_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child) {
     conf->subscriber_first_message = (prev->subscriber_first_message == NCHAN_SUBSCRIBER_FIRST_MESSAGE_UNSET) ? NCHAN_SUBSCRIBER_DEFAULT_FIRST_MESSAGE : prev->subscriber_first_message;
   }
   
-  ngx_conf_merge_sec_value(conf->websocket_ping_interval, prev->websocket_ping_interval, NCHAN_DEFAULT_WEBSOCKET_PING_INTERVAL);
+  MERGE_CONF(conf, prev, subscriber_info_string);
+  if(conf->subscriber_info_string == NULL) { //still null? use the default string
+    if(create_complex_value_from_ngx_str(cf, &conf->subscriber_info_string, &DEFAULT_SUBSCRIBER_INFO_STRING) == NGX_CONF_ERROR) {
+      return NGX_CONF_ERROR;
+    }
+  }
+  
+  ngx_conf_merge_value(conf->subscriber_info_location, prev->subscriber_info_location, 0);
+  
+  ngx_conf_merge_sec_value(conf->websocket_ping_interval, prev->websocket_ping_interval, NCHAN_DEFAULT_SUBSCRIBER_PING_INTERVAL);
+  
+  ngx_conf_merge_sec_value(conf->eventsource_ping.interval, prev->eventsource_ping.interval, NCHAN_DEFAULT_SUBSCRIBER_PING_INTERVAL);
+  ngx_conf_merge_str_value(conf->eventsource_ping.data, prev->eventsource_ping.comment, "");
+  ngx_conf_merge_str_value(conf->eventsource_ping.event, prev->eventsource_ping.event, "ping");
+  ngx_conf_merge_str_value(conf->eventsource_ping.data, prev->eventsource_ping.data, "");
   
   ngx_conf_merge_sec_value(conf->subscriber_timeout, prev->subscriber_timeout, NCHAN_DEFAULT_SUBSCRIBER_TIMEOUT);
   ngx_conf_merge_sec_value(conf->redis_idle_channel_cache_timeout, prev->redis_idle_channel_cache_timeout, NCHAN_DEFAULT_REDIS_IDLE_CHANNEL_CACHE_TIMEOUT);
@@ -389,13 +475,27 @@ static char * nchan_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child) {
     }
     conf->last_message_id.n = 2;
   }
-  
+    
   ngx_conf_merge_value(conf->redis.url_enabled, prev->redis.url_enabled, 0);
+  
   ngx_conf_merge_value(conf->redis.upstream_inheritable, prev->redis.upstream_inheritable, 0);
   ngx_conf_merge_str_value(conf->redis.url, prev->redis.url, NCHAN_REDIS_DEFAULT_URL);
-  ngx_conf_merge_str_value(conf->redis.namespace, prev->redis.namespace, "");
+  
+  if(up && up->redis.namespace.len > 0) { //upstream has a namespace set
+    ngx_conf_merge_str_value(conf->redis.namespace, up->redis.namespace, ""); 
+  }
+  else {
+    ngx_conf_merge_str_value(conf->redis.namespace, prev->redis.namespace, "");
+  }
+  
+  if(up)
+    ngx_conf_merge_value(conf->redis.ping_interval, up->redis.ping_interval, NGX_CONF_UNSET);
   ngx_conf_merge_value(conf->redis.ping_interval, prev->redis.ping_interval, NCHAN_REDIS_DEFAULT_PING_INTERVAL_TIME);
   
+  if(up)
+    ngx_conf_merge_value(conf->redis.nostore_fastpublish, up->redis.nostore_fastpublish, NGX_CONF_UNSET);
+  ngx_conf_merge_value(conf->redis.nostore_fastpublish, prev->redis.nostore_fastpublish, 0);
+  
   if(conf->redis.url_enabled) {
     conf->redis.enabled = 1;
     nchan_store_redis_add_active_loc_conf(cf, conf);
@@ -405,6 +505,8 @@ static char * nchan_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child) {
     ngx_conf_set_redis_upstream(cf, &conf->redis.upstream_url, conf);
   }
   
+  if(up)
+    MERGE_UNSET_CONF(conf->redis.storage_mode, up->redis.storage_mode, REDIS_MODE_CONF_UNSET, REDIS_MODE_CONF_UNSET);
   MERGE_UNSET_CONF(conf->redis.storage_mode, prev->redis.storage_mode, REDIS_MODE_CONF_UNSET, REDIS_MODE_DISTRIBUTED);
   
   if(prev->request_handler != NULL && conf->request_handler == NULL) {
@@ -414,6 +516,14 @@ static char * nchan_merge_loc_conf(ngx_conf_t *cf, void *parent, void *child) {
     nchan_setup_handler(cf, conf->request_handler);
   }
   
+  ngx_conf_merge_value(conf->benchmark.time, prev->benchmark.time, 10);
+  ngx_conf_merge_value(conf->benchmark.msgs_per_minute, prev->benchmark.msgs_per_minute, 120);
+  ngx_conf_merge_value(conf->benchmark.msg_padding, prev->benchmark.msg_padding, 0);
+  ngx_conf_merge_value(conf->benchmark.channels, prev->benchmark.channels, 1000);
+  ngx_conf_merge_value(conf->benchmark.subscribers_per_channel, prev->benchmark.subscribers_per_channel, 100);
+  MERGE_UNSET_CONF(conf->benchmark.subscriber_distribution, prev->benchmark.subscriber_distribution, NCHAN_BENCHMARK_SUBSCRIBER_DISTRIBUTION_UNSET, NCHAN_BENCHMARK_SUBSCRIBER_DISTRIBUTION_RANDOM);
+  MERGE_UNSET_CONF(conf->benchmark.publisher_distribution, prev->benchmark.publisher_distribution, NCHAN_BENCHMARK_PUBLISHER_DISTRIBUTION_UNSET, NCHAN_BENCHMARK_PUBLISHER_DISTRIBUTION_RANDOM);
+  
   return NGX_CONF_OK;
 }
 
@@ -592,6 +702,28 @@ static char *nchan_pubsub_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *co
   return NGX_CONF_OK;
 }
 
+static char *nchan_subscriber_info_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
+  nchan_loc_conf_t     *lcf = conf;
+  nchan_conf_subscriber_types_t *subt = &lcf->sub;
+  
+  // doesn't make sense to have longpoll be the default HTTP subscriber, since channel info locations are likely to be curl'd by developers
+  subt->poll=0;
+  subt->http_raw_stream = 1;
+  subt->longpoll=0;
+  subt->websocket=1;
+  subt->eventsource=1;
+  subt->http_chunked=1;
+  subt->http_multipart=1;
+  
+  lcf->subscriber_info_location = 1;
+  
+  lcf->message_timeout=10;
+  lcf->complex_message_timeout = NULL;
+  
+  lcf->request_handler = &nchan_subscriber_info_handler;
+  return NGX_CONF_OK;
+}
+
 static char *nchan_group_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
   nchan_loc_conf_t     *lcf = conf;
   ngx_str_t            *val;
@@ -634,6 +766,87 @@ static char *nchan_group_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *con
   return NGX_CONF_OK;
 }
 
+static ngx_int_t set_complex_value(ngx_conf_t *cf, ngx_http_complex_value_t **cv, char *val) {
+  ngx_http_compile_complex_value_t    ccv;
+  ngx_str_t                          *value = ngx_palloc(cf->pool, sizeof(ngx_str_t));;
+  if(value == NULL) {
+    return NGX_ERROR;
+  }
+  value->data = (u_char *)val;
+  value->len = strlen(val);
+  *cv = ngx_palloc(cf->pool, sizeof(ngx_http_complex_value_t));
+  if (*cv == NULL) {
+    return NGX_ERROR;
+  } 
+  ngx_memzero(&ccv, sizeof(ngx_http_compile_complex_value_t));
+  ccv.cf = cf;
+  ccv.value = value;
+  ccv.complex_value = *cv;
+  if (ngx_http_compile_complex_value(&ccv) != NGX_OK) {
+    return NGX_ERROR;
+  }
+  
+  return NGX_OK;
+}
+
+static ngx_int_t set_complex_value_array_size1(ngx_conf_t *cf, nchan_complex_value_arr_t *chid, char *val) {
+  chid->n = 1;
+  return set_complex_value(cf, &chid->cv[0], val);
+}
+
+static char *nchan_benchmark_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
+  nchan_loc_conf_t     *lcf = conf;
+  global_benchmark_enabled = 1;
+  lcf->request_handler = &nchan_benchmark_handler;
+  
+  
+  //set group
+  if(set_complex_value(cf, &lcf->channel_group, "benchmark") != NGX_OK) {
+    return "error setting benchmark channel group";
+  }
+  //set pub and sub channel ids
+  if(set_complex_value_array_size1(cf, &lcf->pub_chid, "control") != NGX_OK) {
+    return "error setting benchmark control channel";
+  }
+  if(set_complex_value_array_size1(cf, &lcf->sub_chid, "data") != NGX_OK) {
+    return "error setting benchmark data channel";
+  }
+  
+  lcf->sub.websocket = 1;
+  lcf->pub.websocket = 1;
+  
+  return NGX_CONF_OK;
+}
+
+static char *nchan_benchmark_subscriber_distribution_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
+  ngx_str_t          *val = &((ngx_str_t *) cf->args->elts)[1];
+  nchan_loc_conf_t   *lcf = conf;
+  if(nchan_strmatch(val, 1, "random")) {
+    lcf->benchmark.subscriber_distribution = NCHAN_BENCHMARK_SUBSCRIBER_DISTRIBUTION_RANDOM;
+  }
+  else if(nchan_strmatch(val, 2, "optimal", "best")) {
+    lcf->benchmark.subscriber_distribution = NCHAN_BENCHMARK_SUBSCRIBER_DISTRIBUTION_OPTIMAL;
+  }
+  else {
+    return "invalid value, must be \"random\" or \"optimal\"";
+  }
+  return NGX_CONF_OK;
+}
+static char *nchan_benchmark_publisher_distribution_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
+  ngx_str_t          *val = &((ngx_str_t *) cf->args->elts)[1];
+  nchan_loc_conf_t   *lcf = conf;
+  if(nchan_strmatch(val, 1, "random")) {
+    lcf->benchmark.publisher_distribution = NCHAN_BENCHMARK_PUBLISHER_DISTRIBUTION_RANDOM;
+  }
+  else if(nchan_strmatch(val, 2, "optimal", "best")) {
+    lcf->benchmark.publisher_distribution = NCHAN_BENCHMARK_PUBLISHER_DISTRIBUTION_OPTIMAL;
+  }
+  else {
+    return "invalid value, must be \"random\" or \"optimal\"";
+  }
+  return NGX_CONF_OK;
+}
+
 static char *nchan_subscriber_first_message_directive(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
   nchan_loc_conf_t   *lcf = (nchan_loc_conf_t *)conf;
   ngx_str_t          *val = &((ngx_str_t *) cf->args->elts)[1];
@@ -771,6 +984,9 @@ static char *nchan_conf_deflate_compression_memlevel_directive(ngx_conf_t *cf, n
 
 
 static void nchan_exit_worker(ngx_cycle_t *cycle) {
+  if(!global_nchan_enabled) {
+    return;
+  }
   if(global_redis_enabled) {
     redis_store_prepare_to_exit_worker();
   }
@@ -790,6 +1006,12 @@ static void nchan_exit_worker(ngx_cycle_t *cycle) {
 }
 
 static void nchan_exit_master(ngx_cycle_t *cycle) {
+  if(!global_nchan_enabled) {
+    return;
+  }
+  if(global_benchmark_enabled) {
+    nchan_benchmark_exit_master(cycle);
+  }
   nchan_store_memory.exit_master(cycle);
   if(global_redis_enabled) {
     nchan_store_redis.exit_master(cycle);
@@ -835,11 +1057,13 @@ static char *nchan_set_pub_channel_id(ngx_conf_t *cf, ngx_command_t *cmd, void *
 }
 
 static char *nchan_set_sub_channel_id(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
-  return nchan_set_complex_value_array(cf, cmd, conf, &((nchan_loc_conf_t *)conf)->sub_chid);
+  nchan_loc_conf_t *lcf = conf;
+  return nchan_set_complex_value_array(cf, cmd, conf, &lcf->sub_chid);
 }
 
 static char *nchan_set_pubsub_channel_id(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
-  return nchan_set_complex_value_array(cf, cmd, conf, &((nchan_loc_conf_t *)conf)->pubsub_chid);
+  nchan_loc_conf_t *lcf = conf;
+  return nchan_set_complex_value_array(cf, cmd, conf, &lcf->pubsub_chid);
 }
 
 static char *nchan_subscriber_last_message_id(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
@@ -1096,8 +1320,18 @@ static char *ngx_conf_upstream_redis_server(ngx_conf_t *cf, ngx_command_t *cmd,
   ngx_http_upstream_srv_conf_t        *uscf;
   ngx_str_t                           *value;
   ngx_http_upstream_server_t          *usrv;
-  
+  nchan_loc_conf_t                    *lcf = conf;
   uscf = ngx_http_conf_get_module_srv_conf(cf, ngx_http_upstream_module);  
+  nchan_srv_conf_t                    *scf = NULL;
+  scf = ngx_http_conf_upstream_srv_conf(uscf, ngx_nchan_module);
+  if(scf->upstream_nchan_loc_conf) {
+    assert(scf->upstream_nchan_loc_conf == lcf);
+  }
+  else {
+    //is this even a safe technique? it might break in the future...
+    scf->upstream_nchan_loc_conf = lcf;
+  }
+  
   
   if(uscf->servers == NULL) {
         uscf->servers = ngx_array_create(cf->pool, 4, sizeof(ngx_http_upstream_server_t));
@@ -1122,6 +1356,17 @@ static char *ngx_conf_upstream_redis_server(ngx_conf_t *cf, ngx_command_t *cmd,
   return NGX_CONF_OK;
 }
 
+static char *ngx_conf_set_str_slot_no_newlines(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
+  ngx_str_t *val = cf->args->elts;
+  ngx_str_t *arg = &val[1];
+  
+  if(nchan_ngx_str_substr(arg, "\n")) {
+    return "can't contain any newline characters";
+  }
+  
+  return ngx_conf_set_str_slot(cf, cmd, conf);
+}
+
 static char *ngx_conf_set_redis_url(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
   nchan_loc_conf_t  *lcf = conf;
   ngx_str_t *val = cf->args->elts;
@@ -1139,10 +1384,8 @@ static char *ngx_conf_set_redis_url(ngx_conf_t *cf, ngx_command_t *cmd, void *co
   return ngx_conf_set_str_slot(cf, cmd, conf);
 }
 
-static char *ngx_conf_set_redis_namespace_slot(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
-  //raise(SIGSTOP);
-  ngx_str_t *val = cf->args->elts;
-  ngx_str_t *arg = &val[1];
+static char *ngx_conf_process_redis_namespace_slot(ngx_conf_t *cf, void *post, void *fld) {
+  ngx_str_t *arg = fld;
 
   if(memchr(arg->data, '{', arg->len)) {
     return "can't contain character '{'";
@@ -1154,7 +1397,7 @@ static char *ngx_conf_set_redis_namespace_slot(ngx_conf_t *cf, ngx_command_t *cm
   
   if(arg->len > 0 && arg->data[arg->len-1] != ':') {
     u_char  *nns;
-    if((nns = ngx_palloc(cf->pool, sizeof(arg->len + 2))) == NULL) {
+    if((nns = ngx_palloc(cf->pool, arg->len + 2)) == NULL) {
       return "couldn't allocate redis namespace data";
     }
     ngx_memcpy(nns, arg->data, arg->len);
@@ -1164,7 +1407,7 @@ static char *ngx_conf_set_redis_namespace_slot(ngx_conf_t *cf, ngx_command_t *cm
     arg->data=nns;
   }
   
-  return ngx_conf_set_str_slot(cf, cmd, conf);
+  return NGX_CONF_OK;
 }
 
 static char *ngx_conf_set_redis_storage_mode_slot(ngx_conf_t *cf, ngx_command_t *cmd, void *conf) {
@@ -1186,8 +1429,11 @@ static char *ngx_conf_set_redis_storage_mode_slot(ngx_conf_t *cf, ngx_command_t
   else if(nchan_strmatch(arg, 1, "distributed")) {
     *field = REDIS_MODE_DISTRIBUTED;
   }
+  else if(nchan_strmatch(arg, 1, "nostore") ||  nchan_strmatch(arg, 1, "distributed-nostore")) {
+    *field = REDIS_MODE_DISTRIBUTED_NOSTORE;
+  }
   else {
-    return "is invalid, must be either 'distributed' or 'backup'";
+    return "is invalid, must be one of 'distributed',  'backup' or 'nostore'";
   }
   
   return NGX_CONF_OK;
@@ -1205,7 +1451,7 @@ static char *ngx_conf_set_redis_upstream_pass(ngx_conf_t *cf, ngx_command_t *cmd
 #include "nchan_config_commands.c" //hideous but hey, it works
 
 static ngx_http_module_t  nchan_module_ctx = {
-    nchan_add_variables,           /* preconfiguration */
+    nchan_preconfig,               /* preconfiguration */
     nchan_postconfig,              /* postconfiguration */
     nchan_create_main_conf,        /* create main configuration */
     NULL,                          /* init main configuration */
