@@ -80,8 +80,11 @@ static size_t                     redis_publish_message_msgkey_size;
 #define redis_command(node, cb, pd, fmt, args...)                 \
   do {                                                               \
     if(node->state >= REDIS_NODE_READY) {                            \
-      node->pending_commands++;                                      \
-      nchan_update_stub_status(redis_pending_commands, 1);           \
+      if((cb) != NULL) {                                               \
+        /* a reply is expected, so track this command */             \
+        node->pending_commands++;                                    \
+        nchan_update_stub_status(redis_pending_commands, 1);         \
+      }                                                              \
       redisAsyncCommand((node)->ctx.cmd, cb, pd, fmt, ##args);       \
     } else {                                                         \
       node_log_error(node, "Can't run redis command: no connection to redis server.");\
@@ -242,19 +245,24 @@ static void redis_store_reap_chanhead(rdstore_channel_head_t *ch) {
   DBG("reap channel %V", &ch->id);
 
   if(ch->pubsub_status == REDIS_PUBSUB_SUBSCRIBED) {
-    assert(ch->redis.nodeset->settings.storage_mode == REDIS_MODE_DISTRIBUTED);
+    assert(ch->redis.nodeset->settings.storage_mode >= REDIS_MODE_DISTRIBUTED);
     assert(ch->redis.node.pubsub);
     ch->pubsub_status = REDIS_PUBSUB_UNSUBSCRIBED;
     redis_subscriber_command(ch->redis.node.pubsub, NULL, NULL, "UNSUBSCRIBE %b{channel:%b}:pubsub", STR(ch->redis.nodeset->settings.namespace), STR(&ch->id));
   }
-  
+
+  /*
   redis_nodeset_t *ns = ch->redis.nodeset;
   redis_node_t *cmd = ch->redis.node.cmd;
   redis_node_t *pubsub = ch->redis.node.pubsub;
+  */
   
   nodeset_dissociate_chanhead(ch);
-  rdstore_channel_head_t *cur;
   
+  /*  
+  rdstore_channel_head_t *cur;
+
+
   for(cur = nchan_slist_first(&ns->channels.all); cur != NULL; cur = nchan_slist_next(&ns->channels.all, cur)) {
     assert(cur != ch);
   }
@@ -280,7 +288,7 @@ static void redis_store_reap_chanhead(rdstore_channel_head_t *ch) {
       assert(cur != ch);
     }
   }
-  
+*/  
   
   
   DBG("chanhead %p (%V) is empty and expired. delete.", ch, &ch->id);
@@ -454,7 +462,7 @@ static ngx_int_t log_redis_reply(char *name, ngx_msec_t t) {
   return NGX_OK;
 }
 
-static ngx_int_t redisReply_to_int(redisReply *el, ngx_int_t *integer) {
+static ngx_int_t redisReply_to_ngx_int(redisReply *el, ngx_int_t *integer) {
   if(CHECK_REPLY_INT(el)) {
     *integer=el->integer;
   }
@@ -723,7 +731,7 @@ static void redis_subscriber_callback(redisAsyncContext *c, void *r, void *privd
   
   if(CHECK_REPLY_ARRAY_MIN_SIZE(reply, 3)
    && CHECK_REPLY_STR(reply->element[0])
-   && CHECK_REPLY_STR(reply->element[1])) {
+   && (CHECK_REPLY_STR(reply->element[1]) || CHECK_REPLY_INT(reply->element[1]))) {
     pubsub_channel.data = (u_char *)reply->element[1]->str;
     pubsub_channel.len = reply->element[1]->len;
     chid = get_channel_id_from_pubsub_channel(&pubsub_channel, namespace, &chid_str);
@@ -851,7 +859,6 @@ static void redis_subscriber_callback(redisAsyncContext *c, void *r, void *privd
                 //TODO
               }
               ERR("unsub one not yet implemented");
-              assert(0);
             }
             else if(ngx_strmatch(&alerttype, "unsub all") && array_sz > 1) {
               if(cmp_to_str(&cmp, &extracted_channel_id)) {
@@ -864,28 +871,36 @@ static void redis_subscriber_callback(redisAsyncContext *c, void *r, void *privd
                 //TODO
               }
               ERR("unsub all except not yet  implemented");
-              assert(0);
             }
+            else if(ngx_strmatch(&alerttype, "subscriber info")) {
+              uint64_t request_id;
+              cmp_read_uinteger(&cmp, &request_id);
+              
+              if((chanhead = nchan_store_get_chanhead(chid, nodeset)) == NULL) {
+                ERR("received invalid subscriber info notice with bad channel name");
+              }
+              else {
+                chanhead->spooler.fn->broadcast_notice(&chanhead->spooler, NCHAN_NOTICE_SUBSCRIBER_INFO_REQUEST, (void *)(intptr_t )request_id);
+              }
+            }
+            
             else {
               ERR("unexpected msgpack alert from redis");
-              assert(0);
             }
           }
           else {
             ERR("unexpected msgpack message from redis");
-            assert(0);
           }
         }
         else {
           ERR("unexpected msgpack object from redis");
-          assert(0);
         }
       }
       else {
         ERR("invalid msgpack message from redis: %s", cmp_strerror(&cmp));
-        assert(0);
       }
     }
+
     else { //not a string
       redisEchoCallback(c, el, NULL);
     }
@@ -990,6 +1005,7 @@ void spooler_get_message_finish_handler(channel_spooler_t *spl, void *pd) {
 
 static ngx_int_t start_chanhead_spooler(rdstore_channel_head_t *head) {
   static uint8_t channel_buffer_complete = 1;
+  spooler_fetching_strategy_t spooling_strategy;
   static channel_spooler_handlers_t handlers = {
     spooler_add_handler,
     spooler_dequeue_handler,
@@ -1000,7 +1016,14 @@ static ngx_int_t start_chanhead_spooler(rdstore_channel_head_t *head) {
   };
   nchan_loc_conf_t *lcf = head->redis.nodeset->first_loc_conf; //any loc_conf that refers to this nodeset will work. 
   //the spooler needs it to pass to get_message calls, which in rdstore's case only cares about the nodeset referenced in the loc_conf
-  start_spooler(&head->spooler, &head->id, &head->status, &channel_buffer_complete, &nchan_store_redis, lcf, FETCH, &handlers, head);
+  if(head->redis.nodeset->settings.storage_mode == REDIS_MODE_DISTRIBUTED_NOSTORE) {
+    spooling_strategy = NCHAN_SPOOL_PASSTHROUGH;
+  }
+  else {
+    spooling_strategy = NCHAN_SPOOL_FETCH;
+  }
+  
+  start_spooler(&head->spooler, &head->id, &head->status, &channel_buffer_complete, &nchan_store_redis, lcf, spooling_strategy, &handlers, head);
   return NGX_OK;
 }
 
@@ -1076,6 +1099,13 @@ static void redis_subscriber_register_cb(redisAsyncContext *c, void *vr, void *p
     return; 
   }
   
+  if (!redisReplyOk(c, reply)) {
+    //TODO: fail less silently, maybe retry subscriber registration?
+    sdata->sub->fn->release(sdata->sub, 0);
+    ngx_free(sdata);
+    return;
+  }
+  
   if(  CHECK_REPLY_ARRAY_MIN_SIZE(reply, 4)
     && CHECK_REPLY_INT(reply->element[3])
   ) {
@@ -1083,23 +1113,23 @@ static void redis_subscriber_register_cb(redisAsyncContext *c, void *vr, void *p
     sdata->sub->fn->notify(sdata->sub, NCHAN_NOTICE_REDIS_CHANNEL_MESSAGE_BUFFER_SIZE_CHANGE, (void *)(intptr_t )reply->element[3]->integer);
   }
   
-  sdata->sub->fn->release(sdata->sub, 0);
-  
-  if (!redisReplyOk(c, reply)) {
-    ngx_free(sdata);
-    return;
-  }
-  
-  if ( !CHECK_REPLY_ARRAY_MIN_SIZE(reply, 3) || !CHECK_REPLY_INT(reply->element[1]) || !CHECK_REPLY_INT(reply->element[2])) {
-    //no good
-    redisEchoCallback(c,reply,privdata);
-    return;
-  }
   if(sdata->generation == sdata->chanhead->generation) {
     //is the subscriber     
     //TODO: set subscriber id
     //sdata->sub->id = reply->element[1]->integer;
   }
+  
+  sdata->sub->fn->release(sdata->sub, 0);
+  sdata->sub = NULL; //don't use it anymore, it might have been freed
+  
+  if ( !CHECK_REPLY_ARRAY_MIN_SIZE(reply, 3) || !CHECK_REPLY_INT(reply->element[1]) || !CHECK_REPLY_INT(reply->element[2])) {
+    //no good
+    //TODO: fail less silently, maybe retry subscriber registration?
+    redisEchoCallback(c,reply,privdata);
+    ngx_free(sdata);
+    return;
+  }
+  
   keepalive_ttl = reply->element[2]->integer;
   if(keepalive_ttl > 0) {
     if(!sdata->chanhead->keepalive_timer.timer_set) {
@@ -1273,7 +1303,7 @@ ngx_int_t ensure_chanhead_pubsub_subscribed_if_needed(rdstore_channel_head_t *ch
   redis_node_t       *pubsub_node;
   ngx_str_t          *namespace;
   if( ch->pubsub_status != REDIS_PUBSUB_SUBSCRIBED && ch->pubsub_status != REDIS_PUBSUB_SUBSCRIBING
-   && ch->redis.nodeset->settings.storage_mode == REDIS_MODE_DISTRIBUTED
+   && ch->redis.nodeset->settings.storage_mode >= REDIS_MODE_DISTRIBUTED
    && nodeset_ready(ch->redis.nodeset)
   ) {
     pubsub_node = nodeset_node_pubsub_find_by_chanhead(ch);
@@ -1647,7 +1677,7 @@ static ngx_int_t msg_from_redis_get_message_reply(nchan_msg_t *msg, nchan_compre
   
   redisReply         **els = r->element;
   size_t               content_type_len = 0, es_event_len = 0;
-  ngx_int_t            time_int, ttl;
+  ngx_int_t            time_int = 0, ttl;
   ngx_int_t            compression;
   
   if(CHECK_REPLY_ARRAY_MIN_SIZE(r, offset + 8)
@@ -1672,7 +1702,7 @@ static ngx_int_t msg_from_redis_get_message_reply(nchan_msg_t *msg, nchan_compre
     msg->buf.last_buf = 1;
     msg->buf.last_in_chain = 1;
     
-    if(redisReply_to_int(els[offset], &ttl) != NGX_OK) {
+    if(redisReply_to_ngx_int(els[offset], &ttl) != NGX_OK) {
       ERR("invalid ttl integer value in msg response from redis");
       return NGX_ERROR;
     }
@@ -1684,7 +1714,7 @@ static ngx_int_t msg_from_redis_get_message_reply(nchan_msg_t *msg, nchan_compre
     
     msg->compressed = NULL;
     if(r->elements >= (uint16_t )(offset + 8)) {
-      if(!CHECK_REPLY_INT_OR_STR(els[offset+8]) || redisReply_to_int(els[offset+8], &compression) != NGX_OK) {
+      if(!CHECK_REPLY_INT_OR_STR(els[offset+8]) || redisReply_to_ngx_int(els[offset+8], &compression) != NGX_OK) {
         ERR("invalid compression type integer value in msg response from redis");
         return NGX_ERROR;
       }
@@ -1713,7 +1743,7 @@ static ngx_int_t msg_from_redis_get_message_reply(nchan_msg_t *msg, nchan_compre
       msg->eventsource_event = NULL;
     }
     
-    if(redisReply_to_int(els[offset+1], &time_int) == NGX_OK) {
+    if(redisReply_to_ngx_int(els[offset+1], &time_int) == NGX_OK) {
       msg->id.time = time_int;
     }
     else {
@@ -1721,13 +1751,13 @@ static ngx_int_t msg_from_redis_get_message_reply(nchan_msg_t *msg, nchan_compre
       ERR("invalid msg time from redis");
     }
     
-    redisReply_to_int(els[offset+2], (ngx_int_t *)&msg->id.tag.fixed[0]); // tag is a uint, meh.
+    redisReply_to_ngx_int(els[offset+2], (ngx_int_t *)&msg->id.tag.fixed[0]); // tag is a uint, meh.
     msg->id.tagcount = 1;
     msg->id.tagactive = 0;
     
-    redisReply_to_int(els[offset+3], &time_int);
+    redisReply_to_ngx_int(els[offset+3], &time_int);
     msg->prev_id.time = time_int;
-    redisReply_to_int(els[offset+4], (ngx_int_t *)&msg->prev_id.tag.fixed[0]);
+    redisReply_to_ngx_int(els[offset+4], (ngx_int_t *)&msg->prev_id.tag.fixed[0]);
     msg->prev_id.tagcount = 1;
     msg->prev_id.tagactive = 0;
     
@@ -1904,8 +1934,27 @@ ngx_int_t rdstore_initialize_chanhead_reaper(nchan_reaper_t *reaper, char *name)
   return NGX_OK;
 }
 
+static ngx_int_t nchan_store_init_redis_loc_conf_postconfig(nchan_loc_conf_t *lcf) {
+  nchan_redis_conf_t    *rcf = &lcf->redis;
+  
+  assert(rcf->enabled);
+  
+  //server-scope loc_conf may have some undefined values (because it was never merged with a prev)
+  //thus we must reduntantly check for unser values
+  if(rcf->ping_interval == NGX_CONF_UNSET) {
+    rcf->ping_interval = NCHAN_REDIS_DEFAULT_PING_INTERVAL_TIME;
+  }
+  if(rcf->storage_mode == REDIS_MODE_CONF_UNSET) {
+    rcf->storage_mode = REDIS_MODE_DISTRIBUTED;
+  }
+  if(rcf->nostore_fastpublish == NGX_CONF_UNSET) {
+    rcf->nostore_fastpublish = 0;
+  }
+  
+  return NGX_OK;
+}
+
 static ngx_int_t nchan_store_init_postconfig(ngx_conf_t *cf) {
-  nchan_redis_conf_t    *rcf;
   nchan_loc_conf_t      *lcf;
   nchan_redis_conf_ll_t *cur;
   nchan_main_conf_t     *mcf = ngx_http_conf_get_module_main_conf(cf, ngx_nchan_module);
@@ -1918,19 +1967,9 @@ static ngx_int_t nchan_store_init_postconfig(ngx_conf_t *cf) {
   
   for(cur = redis_conf_head; cur != NULL; cur = cur->next) {
     lcf = cur->lcf;
-    rcf = &lcf->redis;
-    assert(rcf->enabled);
+    nchan_store_init_redis_loc_conf_postconfig(lcf);
     
-      //server-scope loc_conf may have some undefined values (because it was never merged with a prev)
-  //thus we must reduntantly check for unser values
-    if(rcf->ping_interval == NGX_CONF_UNSET) {
-      rcf->ping_interval = NCHAN_REDIS_DEFAULT_PING_INTERVAL_TIME;
-    }
-    if(rcf->storage_mode == REDIS_MODE_CONF_UNSET) {
-      rcf->storage_mode = REDIS_MODE_DISTRIBUTED;
-    }
-    
-    if((nodeset = nodeset_find(rcf)) == NULL) {
+    if((nodeset = nodeset_find(&lcf->redis)) == NULL) {
       nodeset = nodeset_create(lcf);
       rdstore_initialize_chanhead_reaper(&nodeset->chanhead_reaper, "Redis channel reaper");
     }
@@ -1948,6 +1987,7 @@ static void nchan_store_create_main_conf(ngx_conf_t *cf, nchan_main_conf_t *mcf)
   
   //reset redis_conf_head for reloads
   redis_conf_head = NULL;
+  nodeset_destroy_all(); //reset all nodesets before loading config
 }
 
 void redis_store_prepare_to_exit_worker() {
@@ -1992,7 +2032,6 @@ static void nchan_store_exit_worker(ngx_cycle_t *cycle) {
   
   //OLD
   //rbtree_walk(&redis_data_tree, (rbtree_walk_callback_pt )redis_data_tree_exiter_stage2, &chanheads);
-  
   nodeset_destroy_all();
   
   //OLD
@@ -2093,6 +2132,7 @@ typedef struct {
 } redis_publish_callback_data_t;
 
 static void redisPublishCallback(redisAsyncContext *, void *, void *);
+static void redisPublishNostoreCallback(redisAsyncContext *, void *, void *);
 static ngx_int_t redis_publish_message_send(redis_nodeset_t *nodeset, void *pd);
 
 static ngx_int_t redis_publish_message_nodeset_maybe_retry(redis_nodeset_t *ns, redis_publish_callback_data_t *d) {
@@ -2143,22 +2183,90 @@ static ngx_int_t redis_publish_message_send(redis_nodeset_t *nodeset, void *pd)
   }
   d->msglen = msgstr.len;
   
-  
-  
-  //input:  keys: [], values: [namespace, channel_id, time, message, content_type, eventsource_event, compression, msg_ttl, max_msg_buf_size, pubsub_msgpacked_size_cutoff]
-  //output: message_time, message_tag, channel_hash {ttl, time_last_seen, subscribers, messages}
-  nchan_redis_script(publish, node, &redisPublishCallback, d, d->channel_id, 
-                     "%i %b %b %b %i %i %i %i %i", 
-                     msg->id.time, 
-                     STR(&msgstr), 
-                     STR((msg->content_type ? msg->content_type : &empty)), 
-                     STR((msg->eventsource_event ? msg->eventsource_event : &empty)), 
-                     d->compression,
-                     d->message_timeout, 
-                     d->max_messages, 
-                     redis_publish_message_msgkey_size,
-                     nodeset->settings.optimize_target
-                    );
+  if(nodeset->settings.storage_mode == REDIS_MODE_DISTRIBUTED_NOSTORE) {
+    //hand-roll the msgpacked message
+    /*
+    9A A3"msg" CE<ttl> CE<time> 00 00 00 DB<len><str> D9<len><str> D9<len><str> 0X
+    |  |       |       |        |  |  |  |            |            |            |
+    |  |       |       |        |  |  |  |            |            |    fixint0/1 compression
+    |  |       |       |        |  |  |  |            |    bin8 <uint8 len> eventsource-event
+    |  |       |       |        |  |  |  |     bin8 <uint8 len> content type
+    |  |       |       |        |  |  |  bin32 <uint32 BE len> msg data
+    |  |       |       |      fixint=0 tag=0, prev_time=0, prev_tag=0
+    |  |       |    uint32 (BE) message time
+    |  |     uint32 (BE) message ttl
+    |  fixstr[3]
+    fixarray[10]
+    */
+    
+    uint32_t ttl, time;
+    uint32_t msglen;
+    uint8_t  content_type_len, eventsource_event_len, compression;
+    char     zero='\0';
+    int      fastpublish = nodeset->settings.nostore_fastpublish;
+    void   (*publish_callback)(redisAsyncContext *, void *, void *) = NULL;
+    void    *publish_pd = NULL;
+    
+    
+    ttl = htonl(d->message_timeout);
+    time = htonl(msg->id.time);
+    msglen = htonl(d->msglen);
+    content_type_len = msg->content_type ? (msg->content_type->len > 255 ? 255 : msg->content_type->len) : 0;
+    eventsource_event_len = msg->eventsource_event ? (msg->eventsource_event->len > 255 ? 255 : msg->eventsource_event->len) : 0;
+    compression = d->compression;
+    
+    if(!fastpublish) {
+      redis_command(node, NULL, NULL, "MULTI");
+    }
+    else {
+      publish_callback = redisPublishNostoreCallback;
+      publish_pd = d;
+    }
+    redis_command(node, publish_callback, publish_pd,
+      "PUBLISH %b{channel:%b}:pubsub "
+      "\x9A\xA3msg\xCE%b\xCE%b%b%b%b\xDB%b%b\xD9%b%b\xD9%b%b%b",
+      
+      STR(nodeset->settings.namespace),
+      STR(d->channel_id),
+      
+      (char *)&ttl, (size_t )4,
+      (char *)&time, (size_t )4,
+      (char *)&zero, (size_t )1,
+      (char *)&zero, (size_t )1,
+      (char *)&zero, (size_t )1,
+      (char *)&msglen, (size_t )4,
+      STR(&msgstr),
+      (char *)&content_type_len, (size_t )1,
+      STR((msg->content_type ? msg->content_type : &empty)),
+      (char *)&eventsource_event_len, (size_t )1,
+      STR((msg->eventsource_event ? msg->eventsource_event : &empty)),
+      (char *)&compression, (size_t )1
+    );
+    if(!fastpublish) {
+      redis_command(node, NULL, NULL, "HMGET %b{channel:%b} last_seen_fake_subscriber fake_subscribers", 
+        STR(nodeset->settings.namespace),
+        STR(d->channel_id)
+      );
+      redis_command(node, &redisPublishNostoreCallback, d, "EXEC");
+    }
+    
+  }
+  else {  
+    //input:  keys: [], values: [namespace, channel_id, time, message, content_type, eventsource_event, compression, msg_ttl, max_msg_buf_size, pubsub_msgpacked_size_cutoff]
+    //output: message_time, message_tag, channel_hash {ttl, time_last_seen, subscribers, messages}
+    nchan_redis_script(publish, node, &redisPublishCallback, d, d->channel_id, 
+                      "%i %b %b %b %i %i %i %i %i", 
+                      msg->id.time, 
+                      STR(&msgstr), 
+                      STR((msg->content_type ? msg->content_type : &empty)), 
+                      STR((msg->eventsource_event ? msg->eventsource_event : &empty)), 
+                      d->compression,
+                      d->message_timeout, 
+                      d->max_messages, 
+                      redis_publish_message_msgkey_size,
+                      nodeset->settings.optimize_target
+                      );
+  }
   if(mmapped && munmap(msgstr.data, msgstr.len) == -1) {
     ERR("munmap was a problem");
     return NGX_ERROR;
@@ -2194,6 +2302,61 @@ static ngx_int_t nchan_store_publish_message(ngx_str_t *channel_id, nchan_msg_t
   return NGX_OK;
 }
 
+static int64_t redisReply_to_int(redisReply *reply, int nil_value, int wrong_datatype_value) {
+  switch(reply->type) {
+    case REDIS_REPLY_INTEGER:
+      return reply->integer;
+    case REDIS_REPLY_STRING:
+      return atol(reply->str);
+    case REDIS_REPLY_NIL:
+      return nil_value;
+    default:
+      return wrong_datatype_value;
+  }
+}
+
+static void redisPublishNostoreCallback(redisAsyncContext *c, void *r, void *privdata) {
+  redis_publish_callback_data_t *d=(redis_publish_callback_data_t *)privdata;
+  redisReply                    *reply=r;
+  redisReply                   **els;
+  nchan_channel_t                ch;
+  
+  
+  
+  redis_node_t                 *node = c->data;
+  node->pending_commands--;
+  nchan_update_stub_status(redis_pending_commands, -1);
+  
+  if(d->shared_msg) {
+    msg_release(d->msg, "redis publish");
+  }
+  
+  ngx_memzero(&ch, sizeof(ch)); //for debugging basically. should be removed in the future and zeroed as-needed
+  
+  if(reply) {
+    if(reply->type == REDIS_REPLY_ARRAY && reply->elements == 2 && reply->element[1]->type == REDIS_REPLY_ARRAY && reply->element[1]->elements == 2) {
+      els = reply->element[1]->element;
+      ch.last_seen = redisReply_to_int(els[0], 0, 0);
+      ch.subscribers = redisReply_to_int(els[1], 0, 0);
+    }
+    else if(reply->type == REDIS_REPLY_INTEGER) {
+      ch.last_seen = 0;
+      ch.subscribers = redisReply_to_int(reply, 0, 0);
+    }
+    else {
+      DBG("nonsense nostore-publish reply");
+    }
+    
+    d->callback(ch.subscribers > 0 ? NCHAN_MESSAGE_RECEIVED : NCHAN_MESSAGE_QUEUED, &ch, d->privdata);
+  }
+  else {
+    redisEchoCallback(c, r, privdata);
+    d->callback(NGX_HTTP_INTERNAL_SERVER_ERROR, NULL, d->privdata);
+  }
+  
+  ngx_free(d);
+}
+
 static void redisPublishCallback(redisAsyncContext *c, void *r, void *privdata) {
   redis_publish_callback_data_t *d=(redis_publish_callback_data_t *)privdata;
   redisReply                    *reply=r;
@@ -2339,6 +2502,89 @@ int nchan_store_redis_ready(nchan_loc_conf_t *cf) {
   return nodeset && nodeset_ready(nodeset);
 }
 
+
+typedef struct {
+  callback_pt  cb;
+  void        *pd;
+} redis_subscriber_info_id_data_t;
+
+static void get_subscriber_info_id_callback(redisAsyncContext *c, void *r, void *privdata);
+
+static ngx_int_t nchan_store_get_subscriber_info_id(nchan_loc_conf_t *cf, callback_pt cb, void *pd) {
+  redis_nodeset_t  *nodeset = nodeset_find(&cf->redis);
+  
+  if(!nodeset_ready(nodeset)) {
+    return NGX_ERROR;
+  }
+  
+  ngx_str_t request_id_key = ngx_string(NCHAN_REDIS_UNIQUE_REQUEST_ID_KEY);
+  redis_node_t  *node = nodeset_node_find_by_key(nodeset, &request_id_key);
+  if(!node) {
+    return NGX_ERROR;
+  }
+  
+  redis_subscriber_info_id_data_t *d = ngx_alloc(sizeof(*d), ngx_cycle->log);
+  if(d == NULL) {
+    return NGX_ERROR;
+  }
+  
+  d->cb = cb;
+  d->pd = pd;
+  
+  redis_script(get_subscriber_info_id, node, &get_subscriber_info_id_callback, d, "1 %b", STR(&request_id_key));
+  
+  return NGX_DONE;
+}
+
+static void get_subscriber_info_id_callback(redisAsyncContext *c, void *r, void *privdata) {
+  redis_subscriber_info_id_data_t *d = privdata;
+  redisReply                    *reply = r;
+  
+  redis_node_t                 *node = c->data;
+  node->pending_commands--;
+  
+  callback_pt  cb = d->cb;
+  void        *cb_pd = d->pd;
+  
+  ngx_free(d);
+  
+  if (!redisReplyOk(c, reply)) {
+    cb(NGX_ERROR, NULL, cb_pd);
+    return;
+  }
+  
+  int64_t new_id = redisReply_to_int(reply, 0, 0);
+  cb(NGX_OK, (void *)(uintptr_t )new_id, cb_pd);
+}
+
+static ngx_int_t nchan_store_request_subscriber_info(ngx_str_t *channel_id, ngx_int_t request_id, nchan_loc_conf_t *cf, callback_pt cb, void *pd) {
+  if(nchan_channel_id_is_multi(channel_id)) {
+    ERR("redis nchan_store_request_subscriber_info can't handle multi-channel ids");
+    return NGX_ERROR;
+  }
+  
+  redis_nodeset_t               *nodeset = nodeset_find(&cf->redis);
+  if(!nodeset) {
+    ERR("redis nodeset not found for nchan_store_request_subscriber_info");
+    return NGX_ERROR;
+  }
+  
+  if(!nodeset_ready(nodeset)) {
+    ERR("redis nodeset not ready for nchan_store_request_subscriber_info");
+    return NGX_ERROR;
+  }
+  
+  redis_node_t *node = nodeset_node_find_by_channel_id(nodeset, channel_id);
+  if(!node) {
+    ERR("couldn't find Redis node for nchan_store_request_subscriber_info");
+    return NGX_ERROR;
+  }
+  
+  nchan_redis_script( request_subscriber_info, node, &redisCheckErrorCallback, NULL, channel_id, "%i", (int )request_id);
+  
+  return NGX_DONE;
+}
+
 nchan_store_t nchan_store_redis = {
   //init
   &nchan_store_init_module,
@@ -2359,7 +2605,11 @@ nchan_store_t nchan_store_redis = {
   &nchan_store_find_channel, //+callback
   
   NULL, //get_group
-  NULL,
-  NULL
+  NULL, //set group
+  NULL, //delete group
+  
+  &nchan_store_get_subscriber_info_id, //+callback
+  &nchan_store_request_subscriber_info //+callback
+  
 };
 
